@@ -9,22 +9,25 @@ and clearing cached values based on those changes.
 import numpy as np
 
 import zlib
-import time
 import hashlib
 
 from functools import wraps
 
 from .constants import log
-from .util import is_sequence
+from .util import is_sequence, now
+
+try:
+    from collections.abc import Mapping
+except ImportError:
+    from collections import Mapping
 
 try:
     # xxhash is roughly 5x faster than zlib.adler32 but is only
     # packaged in easy wheels on linux (`pip install xxhash`)
     # so we keep it a soft dependency
     import xxhash
-    hasX = True
 except ImportError:
-    hasX = False
+    xxhash = None
 
 
 def tracked_array(array, dtype=None):
@@ -32,7 +35,7 @@ def tracked_array(array, dtype=None):
     Properly subclass a numpy ndarray to track changes.
 
     Avoids some pitfalls of subclassing by forcing contiguous
-    arrays, and does a view into a TrackedArray.
+    arrays and does a view into a TrackedArray.
 
     Parameters
     ------------
@@ -46,9 +49,12 @@ def tracked_array(array, dtype=None):
     tracked : TrackedArray
       Contains input array data
     """
+    # if someone passed us None, just create an empty array
+    if array is None:
+        array = []
     # make sure it is contiguous then view it as our subclass
-    tracked = np.ascontiguousarray(array,
-                                   dtype=dtype).view(TrackedArray)
+    tracked = np.ascontiguousarray(
+        array, dtype=dtype).view(TrackedArray)
     # should always be contiguous here
     assert tracked.flags['C_CONTIGUOUS']
 
@@ -82,26 +88,32 @@ def cache_decorator(function):
         self = args[0]
         # use function name as key in cache
         name = function.__name__
+        # store execution times
+        tic = [now(), 0.0, 0.0]
         # do the dump logic ourselves to avoid
         # verifying cache twice per call
         self._cache.verify()
-        # access cache dict to avoid automatic verification
+        tic[1] = now()
+        # access cache dict to avoid automatic validation
+        # since we already called cache.verify manually
         if name in self._cache.cache:
             # already stored so return value
             return self._cache.cache[name]
-
-        # time execution
-        tic = time.time()
         # value not in cache so execute the function
         value = function(*args, **kwargs)
+        tic[2] = now()
         # store the value
+        if self._cache.force_immutable and hasattr(
+                value, 'flags') and len(value.shape) > 0:
+            value.flags.writeable = False
+
         self._cache.cache[name] = value
-        # debug log execution time
-        # this is nice for debugging as you can see when
-        # cache is getting dumped all the time
-        log.debug('%s was not in cache, executed in %.6f',
+        # log both the function execution time and how long
+        # it took to validate the state of the cache
+        log.debug('`%s`: %.2Es, `cache.verify`: %.2Es',
                   name,
-                  time.time() - tic)
+                  tic[2] - tic[1],
+                  tic[1] - tic[0])
         return value
 
     # all cached values are also properties
@@ -144,13 +156,22 @@ class TrackedArray(np.ndarray):
             obj._modified_m = True
             obj._modified_x = True
 
+    @property
+    def mutable(self):
+        return self.flags['WRITEABLE']
+
+    @mutable.setter
+    def mutable(self, value):
+        self.flags.writeable = value
+
     def md5(self):
         """
         Return an MD5 hash of the current array.
 
         Returns
         -----------
-        md5: str, hexadecimal MD5 of the array
+        md5 : str
+          Hexadecimal MD5 of the array
         """
         if self._modified_m or not hasattr(self, '_hashed_md5'):
             if self.flags['C_CONTIGUOUS']:
@@ -174,7 +195,8 @@ class TrackedArray(np.ndarray):
 
         Returns
         -----------
-        crc: int, checksum from zlib.crc32 or zlib.adler32
+        crc : int
+          Checksum from zlib.crc32 or zlib.adler32
         """
         if self._modified_c or not hasattr(self, '_hashed_crc'):
             if self.flags['C_CONTIGUOUS']:
@@ -195,23 +217,21 @@ class TrackedArray(np.ndarray):
 
         Returns
         -------------
-        xx: int, xxhash.xxh64 hash of array.
+        xx : int
+          xxhash.xxh64 hash of array.
         """
         # repeat the bookkeeping to get a contiguous array inside
         # the function to avoid additional function calls
         # these functions are called millions of times so everything helps
         if self._modified_x or not hasattr(self, '_hashed_xx'):
             if self.flags['C_CONTIGUOUS']:
-                hasher = xxhash.xxh64(self)
-                self._hashed_xx = hasher.intdigest()
+                self._hashed_xx = xxhash.xxh64(self).intdigest()
             else:
                 # the case where we have sliced our nice
                 # contiguous array into a non- contiguous block
                 # for example (note slice *after* track operation):
                 # t = util.tracked_array(np.random.random(10))[::-1]
-                contiguous = np.ascontiguousarray(self)
-                hasher = xxhash.xxh64(contiguous)
-                self._hashed_xx = hasher.intdigest()
+                self._hashed_xx = xxhash.xxh64(np.ascontiguousarray(self)).intdigest()
         self._modified_x = False
         return self._hashed_xx
 
@@ -221,7 +241,8 @@ class TrackedArray(np.ndarray):
 
         Returns
         -----------
-        hash: int, result of fast_hash
+        hash : int
+          Result of fast_hash
         """
         return self.fast_hash()
 
@@ -341,32 +362,39 @@ class TrackedArray(np.ndarray):
         super(self.__class__, self).__setslice__(*args,
                                                  **kwargs)
 
-    if hasX:
-        # if xxhash is installed use it
-        fast_hash = _xxhash
-    else:
+    if xxhash is None:
         # otherwise use our fastest CRC
         fast_hash = crc
+    else:
+        # if xxhash is installed use it
+        fast_hash = _xxhash
 
 
-class Cache:
+class Cache(object):
     """
     Class to cache values which will be stored until the
     result of an ID function changes.
     """
 
-    def __init__(self, id_function):
+    def __init__(self, id_function, force_immutable=False):
         """
         Create a cache object.
 
         Parameters
         ------------
-        id_function: function, that returns hashable value
+        id_function : function
+          Returns hashable value
+        force_immutable : bool
+          If set will make all numpy arrays read-only
         """
         self._id_function = id_function
-
+        # force stored numpy arrays to have flags.writable=False
+        self.force_immutable = bool(force_immutable)
+        # call the id function for initial value
         self.id_current = self._id_function()
+        # a counter for locks
         self._lock = 0
+        # actual store for data
         self.cache = {}
 
     def delete(self, key):
@@ -418,6 +446,11 @@ class Cache:
         checking id_function.
         """
         self.cache.update(items)
+
+        if self.force_immutable:
+            for k, v in self.cache.items():
+                if hasattr(v, 'flags') and len(v.shape) > 0:
+                    v.flags.writeable = False
         self.id_set()
 
     def id_set(self):
@@ -439,6 +472,7 @@ class Cache:
         Returns
         -------------
         cached : object, or None
+          Object that was stored
         """
         self.verify()
         if key in self.cache:
@@ -452,12 +486,18 @@ class Cache:
         Parameters
         ------------
         key : hashable
-                 Key to reference value
+          Key to reference value
         value : any
-                  Value to store in cache
+          Value to store in cache
         """
+        # dumpy cache if ID function has changed
         self.verify()
+        # make numpy arrays read-only if asked to
+        if self.force_immutable and hasattr(value, 'flags') and len(value.shape) > 0:
+            value.flags.writeable = False
+        # assign data to dict
         self.cache[key] = value
+
         return value
 
     def __contains__(self, key):
@@ -476,7 +516,7 @@ class Cache:
         self.id_current = self._id_function()
 
 
-class DataStore:
+class DataStore(Mapping):
     """
     A class to store multiple numpy arrays and track them all
     for changes.
@@ -487,18 +527,43 @@ class DataStore:
     def __init__(self):
         self.data = {}
 
+    def __iter__(self):
+        return iter(self.data)
+
+    def __delitem__(self, key):
+        del self.data[key]
+
     @property
     def mutable(self):
+        """
+        Is data allowed to be altered or not.
+
+        Returns
+        -----------
+        is_mutable : bool
+          Can data be altered in the DataStore
+        """
         if not hasattr(self, '_mutable'):
-            self._mutable = True
+            return True
         return self._mutable
 
     @mutable.setter
     def mutable(self, value):
-        value = bool(value)
-        for i in self.data.value():
-            i.flags.writeable = value
-        self._mutable = value
+        """
+        Is data allowed to be altered or not.
+
+        Parameters
+        ------------
+        is_mutable : bool
+          Should data be allowed to be altered
+        """
+        # make sure passed value is a bool
+        is_mutable = bool(value)
+        # apply the flag to any data stored
+        for n, i in self.data.items():
+            i.mutable = value
+        # save the mutable setting
+        self._mutable = is_mutable
 
     def is_empty(self):
         """
@@ -506,7 +571,8 @@ class DataStore:
 
         Returns
         ----------
-        empty: bool, False if there are items in the DataStore
+        empty : bool
+          False if there are items in the DataStore
         """
         if len(self.data) == 0:
             return True
@@ -527,21 +593,29 @@ class DataStore:
         self.data = {}
 
     def __getitem__(self, key):
-        try:
-            return self.data[key]
-        except KeyError:
-            return np.empty((0,))
+        return self.data[key]
 
     def __setitem__(self, key, data):
         """
         Store an item in the DataStore
         """
+        # we shouldn't allow setting on immutable datastores
+        if not self.mutable:
+            raise ValueError('DataStore is configured immutable!')
+
         if hasattr(data, 'md5'):
             # don't bother to re-track TrackedArray
-            self.data[key] = data
+            tracked = data
         else:
             # otherwise wrap data
-            self.data[key] = tracked_array(data)
+            tracked = tracked_array(data)
+        # apply our mutability setting
+
+        if hasattr(self, '_mutable'):
+            # apply our mutability setting only if it was explicitly set
+            tracked.mutable = self.mutable
+        # store data
+        self.data[key] = tracked
 
     def __contains__(self, key):
         return key in self.data
@@ -549,9 +623,6 @@ class DataStore:
     def __len__(self):
         return len(self.data)
 
-    def values(self):
-        return self.data.values()
-
     def update(self, values):
         if not isinstance(values, dict):
             raise ValueError('Update only implemented for dicts')
@@ -564,7 +635,8 @@ class DataStore:
 
         Returns
         ----------
-        md5: str, MD5 in hexadecimal
+        md5 : str
+          MD5 of data in hexadecimal
         """
         hasher = hashlib.md5()
         for key in sorted(self.data.keys()):
@@ -578,8 +650,10 @@ class DataStore:
 
         Returns
         ----------
-        crc: int, CRC of data
+        crc : int
+          CRC of data
         """
+        # combine with a sum of every hash
         crc = sum(i.crc() for i in self.data.values())
         return crc
 
@@ -589,13 +663,15 @@ class DataStore:
 
         Returns
         ------------
-        hashed: int, checksum of data
+        hashed : int
+          Checksum of data
         """
+        # combine every hash
         fast = sum(i.fast_hash() for i in self.data.values())
         return fast
 
 
-def _fast_crc(count=50):
+def _fast_crc(count=25):
     """
     On certain platforms/builds zlib.adler32 is substantially
     faster than zlib.crc32, but it is not consistent across
@@ -606,20 +682,24 @@ def _fast_crc(count=50):
 
     Parameters
     ------------
-    count: int, number of repetitions to do on the speed trial
+    count : int
+      Number of repetitions to do on the speed trial
 
     Returns
     ----------
-    crc32: function, either zlib.adler32 or zlib.crc32
+    crc32 : function
+      Either `zlib.adler32` or `zlib.crc32`
     """
     import timeit
 
+    # create an array of random numbers
     setup = 'import numpy, zlib;'
     setup += 'd = numpy.random.random((500,3));'
-
+    # time crc32
     crc32 = timeit.timeit(setup=setup,
                           stmt='zlib.crc32(d)',
                           number=count)
+    # time adler32
     adler32 = timeit.timeit(setup=setup,
                             stmt='zlib.adler32(d)',
                             number=count)
