@@ -11,37 +11,42 @@ import collections
 from .. import util
 from .. import visual
 from .. import grouping
+from .. import resources
 
 from ..constants import log
-from ..resources import get_resource
 
 try:
-    import PIL
+    import PIL.Image
 except ImportError:
     pass
 
 # from ply specification, and additional dtypes found in the wild
-ply_dtypes = {'char': 'i1',
-              'uchar': 'u1',
-              'short': 'i2',
-              'ushort': 'u2',
-              'int': 'i4',
-              'int8': 'i1',
-              'int16': 'i2',
-              'int32': 'i4',
-              'uint': 'u4',
-              'uint8': 'u1',
-              'uint16': 'u2',
-              'uint32': 'u4',
-              'float': 'f4',
-              'float16': 'f2',
-              'float32': 'f4',
-              'double': 'f8'}
+dtypes = {
+    'char': 'i1',
+    'uchar': 'u1',
+    'short': 'i2',
+    'ushort': 'u2',
+    'int': 'i4',
+    'int8': 'i1',
+    'int16': 'i2',
+    'int32': 'i4',
+    'int64': 'i8',
+    'uint': 'u4',
+    'uint8': 'u1',
+    'uint16': 'u2',
+    'uint32': 'u4',
+    'uint64': 'u8',
+    'float': 'f4',
+    'float16': 'f2',
+    'float32': 'f4',
+    'float64': 'f8',
+    'double': 'f8'}
 
 
 def load_ply(file_obj,
              resolver=None,
              fix_texture=True,
+             prefer_color=None,
              *args,
              **kwargs):
     """
@@ -57,6 +62,8 @@ def load_ply(file_obj,
       If True, will re- index vertices and faces
       so vertices with different UV coordinates
       are disconnected.
+    prefer_color : None, 'vertex', or 'face'
+      Which kind of color to prefer if both defined
 
     Returns
     ---------
@@ -86,7 +93,8 @@ def load_ply(file_obj,
 
     kwargs = elements_to_kwargs(elements,
                                 fix_texture=fix_texture,
-                                image=image)
+                                image=image,
+                                prefer_color=prefer_color)
 
     return kwargs
 
@@ -127,7 +135,7 @@ def export_ply(mesh,
     dtype_color = ('rgba', '<u1', (4))
 
     # get template strings in dict
-    templates = json.loads(get_resource('ply.template'))
+    templates = json.loads(resources.get('ply.template'))
     # start collecting elements into a string for the header
     header = templates['intro']
     header += templates['vertex']
@@ -260,7 +268,7 @@ def parse_header(file_obj):
             if len(line) == 3:
                 dtype, field = line[1:]
                 elements[name]['properties'][
-                    str(field)] = endian + ply_dtypes[dtype]
+                    str(field)] = endian + dtypes[dtype]
             # is the property a painful list, like:
             # `property list uchar int vertex_indices`
             elif 'list' in line[1]:
@@ -268,10 +276,10 @@ def parse_header(file_obj):
                 elements[name]['properties'][
                     str(field)] = (
                     endian +
-                    ply_dtypes[dtype_count] +
+                    dtypes[dtype_count] +
                     ', ($LIST,)' +
                     endian +
-                    ply_dtypes[dtype])
+                    dtypes[dtype])
         # referenced as a file name
         elif 'TextureFile' in line:
             # textures come listed like:
@@ -283,19 +291,31 @@ def parse_header(file_obj):
     return elements, is_ascii, image_name
 
 
-def elements_to_kwargs(elements, fix_texture, image):
+def elements_to_kwargs(elements,
+                       fix_texture,
+                       image,
+                       prefer_color=None):
     """
     Given an elements data structure, extract the keyword
     arguments that a Trimesh object constructor will expect.
 
     Parameters
     ------------
-    elements: OrderedDict object, with fields and data loaded
+    elements : OrderedDict object
+      With fields and data loaded
+    fix_texture : bool
+      If True, will re- index vertices and faces
+      so vertices with different UV coordinates
+      are disconnected.
+    image : PIL.Image
+      Image to be viewed
+    prefer_color : None, 'vertex', or 'face'
+      Which kind of color to prefer if both defined
 
     Returns
     -----------
-    kwargs: dict, with keys for Trimesh constructor.
-            eg: mesh = trimesh.Trimesh(**kwargs)
+    kwargs : dict
+      Keyword arguments for Trimesh constructor
     """
 
     kwargs = {'metadata': {'ply_raw': elements}}
@@ -353,65 +373,80 @@ def elements_to_kwargs(elements, fix_texture, image):
             # incorrectly is a ValueError
             pass
 
-    # PLY stores texture coordinates per- face which is
-    # slightly annoying, as we have to then figure out
-    # which vertices have the same position but different UV
-    expected = (faces.shape[0], faces.shape[1] * 2)
-    if (image is not None and
-        texcoord is not None and
-            texcoord.shape == expected):
+    if faces is not None:
+        # PLY stores texture coordinates per- face which is
+        # slightly annoying, as we have to then figure out
+        # which vertices have the same position but different UV
+        expected = (faces.shape[0], faces.shape[1] * 2)
+        if (image is not None and
+            texcoord is not None and
+                texcoord.shape == expected):
 
-        # vertices with the same position but different
-        # UV coordinates can't be merged without it
-        # looking like it went through a woodchipper
-        # in- the- wild PLY comes with things merged that
-        # probably shouldn't be so disconnect vertices
-        if fix_texture:
-            # reshape to correspond with flattened faces
-            uv = texcoord.reshape((-1, 2))
+            # vertices with the same position but different
+            # UV coordinates can't be merged without it
+            # looking like it went through a woodchipper
+            # in- the- wild PLY comes with things merged that
+            # probably shouldn't be so disconnect vertices
+            if fix_texture:
+                # do import here
+                from ..visual.texture import unmerge_faces
 
-            # round UV to OOM 10^4 as they are pixel coordinates
-            # and more precision is not necessary or desirable
-            search = np.column_stack((
-                vertices[faces.reshape(-1)],
-                (uv * 1e4).round()))
+                # reshape to correspond with flattened faces
+                uv_all = texcoord.reshape((-1, 2))
+                # UV coordinates defined for every triangle have
+                # duplicates which can be merged so figure out
+                # which UV coordinates are the same here
+                unique, inverse = grouping.unique_rows(uv_all)
 
-            # find vertices which have the same position AND UV
-            unique, inverse = grouping.unique_rows(search)
-
-            # set vertices, faces, and UV to the new values
-            vertices = search[:, :3][unique]
-            faces = inverse.reshape((-1, 3))
-            uv = uv[unique]
-        else:
-            # don't alter vertices, UV will look like crap
-            # if it was exported with vertices merged
-            uv = np.zeros((len(vertices), 2))
-            uv[faces.reshape(-1)] = texcoord.reshape((-1, 2))
-
-        # create the visuals object for the texture
-        kwargs['visual'] = visual.texture.TextureVisuals(
-            uv=uv, image=image)
+                # use the indices of faces and face textures
+                # to only merge vertices where the position
+                # AND uv coordinate are the same
+                faces, mask_v, mask_vt = unmerge_faces(
+                    faces, inverse.reshape(faces.shape))
+                # apply the mask to get resulting vertices
+                vertices = vertices[mask_v]
+                # apply the mask to get UV coordinates
+                uv = uv_all[unique][mask_vt]
+            else:
+                # don't alter vertices, UV will look like crap
+                # if it was exported with vertices merged
+                uv = np.zeros((len(vertices), 2))
+                uv[faces.reshape(-1)] = texcoord.reshape((-1, 2))
 
+            # create the visuals object for the texture
+            kwargs['visual'] = visual.texture.TextureVisuals(
+                uv=uv, image=image)
+        # faces were not none so assign them
+        kwargs['faces'] = faces
     # kwargs for Trimesh or PointCloud
-    kwargs.update({'faces': faces,
-                   'vertices': vertices})
+    kwargs['vertices'] = vertices
 
     # if both vertex and face color are defined pick the one
-    # with the most going on
+    # with the most "signal," i.e. which one is not all zeros
     colors = []
     signal = []
-    if kwargs['faces'] is not None:
+    if faces is not None:
+        # extract face colors or None
         f_color, f_signal = element_colors(elements['face'])
         colors.append({'face_colors': f_color})
         signal.append(f_signal)
-    if kwargs['vertices'] is not None:
+        # extract vertex colors or None
         v_color, v_signal = element_colors(elements['vertex'])
         colors.append({'vertex_colors': v_color})
         signal.append(v_signal)
 
-    # add the winning colors to the result
-    kwargs.update(colors[np.argmax(signal)])
+        if prefer_color is None:
+            # if we are in "auto-pick" mode take the one with the
+            # largest  standard deviation of colors
+            kwargs.update(colors[np.argmax(signal)])
+        elif 'vert' in prefer_color and v_color is not None:
+            # vertex colors are preferred and defined
+            kwargs['vertex_colors'] = v_color
+        elif 'face' in prefer_color and f_color is not None:
+            # face colors are preferred and defined
+            kwargs['face_colors'] = f_color
+    else:
+        kwargs['colors'] = element_colors(elements['vertex'])
 
     return kwargs
 
@@ -436,12 +471,81 @@ def element_colors(element):
 
     if len(candidate_colors) >= 3:
         colors = np.column_stack(candidate_colors)
-        signal = colors.ptp(axis=0).sum()
+        signal = colors.std(axis=0).sum()
         return colors, signal
 
     return None, 0.0
 
 
+def load_element_different(properties, data):
+    """
+    Load elements which include lists of different lengths
+    based on the element's property-definitions.
+
+    Parameters
+    ------------
+    properties : dict
+      Property definitions encoded in a dict where the property name is the key
+      and the property data type the value.
+    data : array
+      Data rows for this element.
+    """
+    element_data = {k: [] for k in properties.keys()}
+    for row in data:
+        start = 0
+        for name, dt in properties.items():
+            length = 1
+            if '$LIST' in dt:
+                dt = dt.split('($LIST,)')[-1]
+                # the first entry in a list-property is the number of elements in the list
+                length = int(row[start])
+                # skip the first entry (the length), when reading the data
+                start += 1
+            end = start + length
+            element_data[name].append(row[start:end].astype(dt))
+            # start next property at the end of this one
+            start = end
+
+    # convert all property lists to numpy arrays
+    for name in element_data.keys():
+        element_data[name] = np.array(element_data[name]).squeeze()
+
+    return element_data
+
+
+def load_element_single(properties, data):
+    """
+    Load element data with lists of a single length
+    based on the element's property-definitions.
+
+    Parameters
+    ------------
+    properties : dict
+      Property definitions encoded in a dict where the property name is the key
+      and the property data type the value.
+    data : array
+      Data rows for this element. If the data contains list-properties,
+      all lists belonging to one property must have the same length.
+    """
+    col_ranges = []
+    start = 0
+    row0 = data[0]
+    for name, dt in properties.items():
+        length = 1
+        if '$LIST' in dt:
+            # the first entry in a list-property is the number of elements in the list
+            length = int(row0[start])
+            # skip the first entry (the length), when reading the data
+            start += 1
+        end = start + length
+        col_ranges.append((start, end))
+        # start next property at the end of this one
+        start = end
+
+    return {n: data[:, c[0]:c[1]].astype(dt.split('($LIST,)')[-1])
+            for c, (n, dt) in zip(col_ranges, properties.items())}
+
+
 def ply_ascii(elements, file_obj):
     """
     Load data from an ASCII PLY file into an existing elements data structure.
@@ -466,45 +570,41 @@ def ply_ascii(elements, file_obj):
                       for i in lines])
 
     # store the line position in the file
-    position = 0
+    row_pos = 0
 
     # loop through data we need
     for key, values in elements.items():
-        # will store (start, end) column index of data
-        columns = collections.deque()
-        # will store the total number of rows
-        rows = 0
+        # if the element is empty ignore it
+        if 'length' not in values or values['length'] == 0:
+            continue
 
-        for name, dtype in values['properties'].items():
-            if '$LIST' in dtype:
-                # if an element contains a list property handle it here
+        data = array[row_pos:row_pos + values['length']]
+        row_pos += values['length']
 
-                row = array[position]
-                list_count = int(row[rows])
+        # try stacking the data, which simplifies column-wise access. this is only
+        # possible, if all rows have the same length.
+        try:
+            data = np.vstack(data)
+            col_count_equal = True
+        except ValueError:
+            col_count_equal = False
 
-                # ignore the count and take the data
-                columns.append([rows + 1,
-                                rows + 1 + list_count])
-                rows += list_count + 1
-                # change the datatype to just the dtype for data
+        # number of list properties in this element
+        list_count = sum(1 for dt in values['properties'].values() if '$LIST' in dt)
+        if col_count_equal and list_count <= 1:
+            # all rows have the same length and we only have at most one list
+            # property where all entries have the same length. this means we can
+            # use the quick numpy-based loading.
+            element_data = load_element_single(
+                values['properties'], data)
+        else:
+            # there are lists of differing lengths. we need to fall back to loading
+            # the data by iterating all rows and checking for list-lengths. this is
+            # slower than the variant above.
+            element_data = load_element_different(
+                values['properties'], data)
 
-                values['properties'][name] = dtype.split('($LIST,)')[-1]
-            else:
-                # a single column data field
-                columns.append([rows, rows + 1])
-                rows += 1
-
-        # get the lines as a 2D numpy array
-        data = np.vstack(array[position:position + values['length']])
-        # offset position in file
-        position += values['length']
-
-        # store columns we care about by name and convert to data type
-        elements[key]['data'] = {n: data[:, c[0]:c[1]].astype(dt)
-                                 for n, dt, c in zip(
-            values['properties'].keys(),    # field name
-            values['properties'].values(),  # data type of field
-            columns)}                       # list of (start, end) column indexes
+        elements[key]['data'] = element_data
 
 
 def ply_binary(elements, file_obj):
@@ -513,11 +613,13 @@ def ply_binary(elements, file_obj):
 
     Parameters
     ------------
-    elements: OrderedDict object, populated from the file header.
-              object will be modified to add data by this function.
+    elements : OrderedDict
+      Populated from the file header.
+      Object will be modified to add data by this function.
 
-    file_obj: open file object, with current position at the start
-              of the data section (past the header)
+    file_obj : open file object
+      With current position at the start
+      of the data section (past the header)
     """
 
     def populate_listsize(file_obj, elements):
@@ -530,10 +632,12 @@ def ply_binary(elements, file_obj):
         """
         p_start = file_obj.tell()
         p_current = file_obj.tell()
+        elem_pop = []
         for element_key, element in elements.items():
             props = element['properties']
             prior_data = ''
             for k, dtype in props.items():
+                prop_pop = []
                 if '$LIST' in dtype:
                     # every list field has two data types:
                     # the list length (single value), and the list data (multiple)
@@ -544,13 +648,32 @@ def ply_binary(elements, file_obj):
                     else:
                         offset = np.dtype(prior_data).itemsize
                     file_obj.seek(p_current + offset)
-                    size = np.frombuffer(file_obj.read(field_dtype.itemsize),
-                                         dtype=field_dtype)[0]
+                    blob = file_obj.read(field_dtype.itemsize)
+                    if len(blob) == 0:
+                        # no data was read for property
+                        prop_pop.append(k)
+                        break
+                    size = np.frombuffer(blob, dtype=field_dtype)[0]
                     props[k] = props[k].replace('$LIST', str(size))
                 prior_data += props[k] + ','
+            if len(prop_pop) > 0:
+                # if a property was empty remove it
+                for pop in prop_pop:
+                    props.pop(pop)
+                # if we've removed all properties from
+                # an element remove the element later
+                if len(props) == 0:
+                    elem_pop.append(element_key)
+                    continue
+            # get the size of the items in bytes
             itemsize = np.dtype(', '.join(props.values())).itemsize
+            # offset the file based on read size
             p_current += element['length'] * itemsize
+        # move the file back to where we found it
         file_obj.seek(p_start)
+        # if there were elements without properties remove them
+        for pop in elem_pop:
+            elements.pop(pop)
 
     def populate_data(file_obj, elements):
         """
@@ -597,7 +720,7 @@ def ply_binary(elements, file_obj):
     populate_data(file_obj, elements)
 
 
-def export_draco(mesh):
+def export_draco(mesh, bits=28):
     """
     Export a mesh using Google's Draco compressed format.
 
@@ -607,6 +730,10 @@ def export_draco(mesh):
     Parameters
     ----------
     mesh : Trimesh object
+      Mesh to export
+    bits : int
+      Bits of quantization for position
+      tol.merge=1e-8 is roughly 25 bits
 
     Returns
     ----------
@@ -618,10 +745,8 @@ def export_draco(mesh):
         temp_ply.flush()
         with tempfile.NamedTemporaryFile(suffix='.drc') as encoded:
             subprocess.check_output([draco_encoder,
-                                     '-qp',  # bits of quantization for position
-                                     '28',  # since our tol.merge is 1e-8, 25 bits
-                                            # more has a machine epsilon
-                                            # smaller than that
+                                     '-qp',
+                                     str(int(bits)),
                                      '-i',
                                      temp_ply.name,
                                      '-o',
@@ -637,7 +762,7 @@ def load_draco(file_obj, **kwargs):
 
     Parameters
     ----------
-    file_obj  : file- like object
+    file_obj : file- like object
       Contains data
 
     Returns
